@@ -39,6 +39,7 @@ typedef const void * gconstpointer;
 typedef uint8_t guint8;
 typedef guint32 GQuark;
 typedef guint32 gunichar;
+typedef signed long gssize;
 
 typedef guint (*GHashFunc)(gconstpointer);
 typedef int (*GCompareFunc)(gconstpointer, gconstpointer);
@@ -116,6 +117,7 @@ typedef struct GOptionContext {
 #define g_malloc malloc
 #define g_free free
 #define g_realloc realloc
+#define g_memdup2 g_memdup
 
 #define	G_STR_DELIMITERS "_-|> <."
 
@@ -143,6 +145,8 @@ void g_printerr(const gchar *format, ...);
 gint g_unichar_to_utf8(gunichar c, gchar *dst);
 gchar *g_locale_to_utf8(const gchar *opsysstring, size_t len,
         size_t *bytes_read, size_t *bytes_written, GError **error);
+gchar *g_utf8_casefold(const gchar *str, gssize len);
+gchar *g_utf8_strdown(const gchar *str, gssize len);
 
 /* GString */
 GString *g_string_new(const gchar *init);
