@@ -236,7 +236,7 @@ mdb_read_indices(MdbTableDef *table)
 	 */
 	//fprintf(stderr, "num_idxs:%d num_real_idxs:%d\n", table->num_idxs, table->num_real_idxs);
 
-	table->num_real_idxs = 0;
+	unsigned int num_idxs_type_other_than_2 = 0;
 	tmpbuf = g_malloc(idx2_sz);
 	for (i=0;i<table->num_idxs;i++) {
 		if (!read_pg_if_n(mdb, tmpbuf, &cur_pos, idx2_sz)) {
@@ -264,8 +264,11 @@ mdb_read_indices(MdbTableDef *table)
 			fprintf(stderr, "idx #%d: %d %d %d %d %d/%d\n", i, idx_marker, rel_idx_type, rel_idx_number, rel_idx_page, update_action_flags, delete_action_flags);
 		}*/
 		if (pidx->index_type!=2)
-			table->num_real_idxs++;
+			num_idxs_type_other_than_2++;
 	}
+	if (num_idxs_type_other_than_2<table->num_real_idxs)
+		table->num_real_idxs=num_idxs_type_other_than_2;
+
 	//fprintf(stderr, "num_idxs:%d num_real_idxs:%d\n", table->num_idxs, table->num_real_idxs);
 	g_free(tmpbuf);
 
@@ -1014,7 +1017,7 @@ int mdb_index_compute_cost(MdbTableDef *table, MdbIndex *idx)
 
 	/*
 	 * a like with a wild card first is useless as a sarg */
-	if (sarg->op == MDB_LIKE && sarg->value.s[0]=='%')
+	if ((sarg->op == MDB_LIKE || sarg->op == MDB_ILIKE) && sarg->value.s[0]=='%')
 		return 0;
 
 	/*
@@ -1027,6 +1030,7 @@ int mdb_index_compute_cost(MdbTableDef *table, MdbIndex *idx)
 				case MDB_EQUAL:
 					return 1; break;
 				case MDB_LIKE:
+				case MDB_ILIKE:
 					return 4; break;
 				case MDB_ISNULL:
 					return 12; break;
@@ -1040,6 +1044,7 @@ int mdb_index_compute_cost(MdbTableDef *table, MdbIndex *idx)
 					else return 1;
 					break;
 				case MDB_LIKE:
+				case MDB_ILIKE:
 					return 6; break;
 				case MDB_ISNULL:
 					return 12; break;
@@ -1053,6 +1058,7 @@ int mdb_index_compute_cost(MdbTableDef *table, MdbIndex *idx)
 				case MDB_EQUAL:
 					return 2; break;
 				case MDB_LIKE:
+				case MDB_ILIKE:
 					return 5; break;
 				case MDB_ISNULL:
 					return 12; break;
@@ -1066,6 +1072,7 @@ int mdb_index_compute_cost(MdbTableDef *table, MdbIndex *idx)
 					else return 2;
 					break;
 				case MDB_LIKE:
+				case MDB_ILIKE:
 					return 7; break;
 				case MDB_ISNULL:
 					return 12; break;
