@@ -81,12 +81,14 @@ extern int yydebug;
     PERCENT = 282,                 /* PERCENT  */
     LTEQ = 283,                    /* LTEQ  */
     GTEQ = 284,                    /* GTEQ  */
-    LIKE = 285,                    /* LIKE  */
-    IS = 286,                      /* IS  */
-    NUL = 287,                     /* NUL  */
-    EQ = 288,                      /* EQ  */
-    LT = 289,                      /* LT  */
-    GT = 290                       /* GT  */
+    NEQ = 285,                     /* NEQ  */
+    LIKE = 286,                    /* LIKE  */
+    ILIKE = 287,                   /* ILIKE  */
+    IS = 288,                      /* IS  */
+    NUL = 289,                     /* NUL  */
+    EQ = 290,                      /* EQ  */
+    LT = 291,                      /* LT  */
+    GT = 292                       /* GT  */
   };
   typedef enum yytokentype yytoken_kind_t;
 #endif
@@ -101,7 +103,7 @@ union YYSTYPE
 	double dval;
 	int ival;
 
-#line 105 "parser.h"
+#line 107 "parser.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
