@@ -17,6 +17,7 @@
  */
 
 #include "mdbtools.h"
+#include "mdbver.h"
 
 void dump_kkd(MdbHandle *mdb, void *kkd, size_t len);
 
@@ -25,21 +26,52 @@ main(int argc, char **argv)
 {
 	MdbHandle *mdb;
 	MdbTableDef *table;
+	char *table_name = NULL;
+	char *locale = NULL;
 	char *name;
 	gchar *propColName;
 	void *buf;
 	int col_num;
 	int found = 0;
+	int print_mdbver = 0;
+    GError *error = NULL;
 
-	if (argc < 3) {
-		fprintf(stderr,"Usage: %s <file> <object name> [<prop col>]\n",
-			argv[0]);
+	GOptionContext *opt_context;
+	GOptionEntry entries[] = {
+		{"version", 0, 0, G_OPTION_ARG_NONE, &print_mdbver, "Show mdbtools version and exit", NULL},
+		{NULL}
+	};
+	opt_context = g_option_context_new("<file> <object name> [<prop col>] - display properties of an object in an Access database");
+	g_option_context_add_main_entries(opt_context, entries, NULL /*i18n*/);
+	locale = setlocale(LC_CTYPE, "");
+	if (!g_option_context_parse (opt_context, &argc, &argv, &error))
+	{
+		fprintf(stderr, "option parsing failed: %s\n", error->message);
+		fputs(g_option_context_get_help(opt_context, TRUE, NULL), stderr);
 		return 1;
 	}
+	if (print_mdbver) {
+		if (argc > 1) {
+			fputs(g_option_context_get_help(opt_context, TRUE, NULL), stderr);
+		}
+		fprintf(stdout,"%s\n", MDB_FULL_VERSION);
+		exit(argc > 1);
+	}
+	if (argc != 3 && argc != 4) {
+		fputs("Wrong number of arguments.\n\n", stderr);
+		fputs(g_option_context_get_help(opt_context, TRUE, NULL), stderr);
+		return 1;
+	}
+
+	table_name = g_locale_to_utf8(argv[2], -1, NULL, NULL, NULL);
 	if (argc < 4)
-		propColName = "LvProp";
+		propColName = g_strdup("LvProp");
 	else
-		propColName = argv[3];
+		propColName = g_locale_to_utf8(argv[3], -1, NULL, NULL, NULL);
+	setlocale(LC_CTYPE, locale);
+	if (!table_name || !propColName) {
+		return 1;
+	}
 
 	mdb = mdb_open(argv[1], MDB_NOFLAGS);
 	if (!mdb) {
@@ -48,6 +80,8 @@ main(int argc, char **argv)
 
 	table = mdb_read_table_by_name(mdb, "MSysObjects", MDB_ANY);
 	if (!table) {
+		g_free(table_name);
+		g_free(propColName);
 		mdb_close(mdb);
 		return 1;
 	}
@@ -62,13 +96,15 @@ main(int argc, char **argv)
 		g_free(name);
 		g_free(buf);
 		mdb_free_tabledef(table);
+		g_free(table_name);
 		mdb_close(mdb);
-		printf("Column %s not found in MSysObjects!\n", argv[3]);
+		printf("Column %s not found in MSysObjects!\n", propColName);
+		g_free(propColName);
 		return 1;
 	}
 
 	while(mdb_fetch_row(table)) {
-		if (!strcmp(name, argv[2])) {
+		if (!strcmp(name, table_name)) {
 			found = 1;
 			break;
 		}
@@ -89,8 +125,15 @@ main(int argc, char **argv)
 	g_free(buf);
 	mdb_free_tabledef(table);
 	mdb_close(mdb);
+	g_free(table_name);
+
+	if (!found) {
+		printf("Object %s not found in database file!\n", propColName);
+	}
 	
-	return 0;
+	g_free(propColName);
+
+	return !found;
 }
 void dump_kkd(MdbHandle *mdb, void *kkd, size_t len)
 {
